# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2011
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################

"""
Agent zephir pour la surveillance ds démons bacula via check_bacula
"""

from twisted.internet import defer
from twisted.internet.utils import getProcessOutput, getProcessValue
from os import system
import traceback

from zephir.monitor.agentmanager.agent import Agent
from zephir.monitor.agentmanager.data import HTMLData, TableData
from zephir.monitor.agentmanager.util import status_to_img, boolean_to_onoff, log
from zephir.monitor.agentmanager import status

class BaculaService(Agent):

    def __init__(self, name, password, **params):
        Agent.__init__(self, name, **params)
        self.name = name
        self.short_name = self.name.split('-')[1]
        self.password = password
        self.table = TableData([
            ('status', "Etat", {'align':'center'}, status_to_img),
            ('description', "Description", {'align':'left'}, None),
            ('host', "Hôte", {'align':'left'}, None),
            ('port', "Port", {'align':'left'}, None) ])
        self.data = [self.table]

    def measure(self):
        CHECK_BACULA = '/usr/lib/nagios/plugins/check_bacula'
        result = getProcessOutput(CHECK_BACULA,
                                args=('-H', 'localhost',
                                    '-D', self.short_name,
                                    '-M', '127.0.0.1-mon',
                                    '-K', self.password),
                                env={'LC_ALL': 'C'})
        result.addCallbacks(self.measure_activity, self.measure_error)
        return result

    def measure_activity(self, data):
        if 'BACULA OK' in str(data):
            status = boolean_to_onoff(1)
        else:
            status = boolean_to_onoff(0)
        mesures=[{ 'description': self.name,
                   'host': 'localhost',
                   'port': '',
                   'status': status }]
        return {'services': mesures}

    def measure_error(self, failure):
        log.msg(u'Erreur remontée par /usr/lib/nagios/plugins/check_bacula :')
        log.msg(failure.getErrorMessage())

    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value['services']

    def check_status(self):
        """remonte une erreur si un des services est tombé"""
        if self.last_measure is not None:
            for service in self.last_measure.value['services']:
                if service['status'] != 'On':
                    return status.Error()
        else:
            # pas de mesure connue
            return status.Unknown()
        return status.OK()
