# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################

"""
Agent zephir pour la surveillance de la crl agriates
"""

from zephir.monitor.agentmanager.agent import Agent
from zephir.monitor.agentmanager.data import HTMLData, TableData
from zephir.monitor.agentmanager import status, util
from twisted.internet.utils import getProcessOutput
import datetime

class Crl(Agent):
    def __init__(self, name, **params):
        Agent.__init__(self, name, **params)
        self.status = status.OK()
        self.table = TableData([
            ('expire', "Date d'expiration", {'align':'left'}, None),
            ])
        title = HTMLData("<h3>Surveillance de la CRL</h3>")
        self.data = [title, self.table]

    def measure(self):
        self.status = status.OK()
        cmd = "/usr/sbin/ipsec"
        args = ["listcrls"]
        update = getProcessOutput(cmd, args)
        return update.addCallbacks(self.callback_expire, self.errback_expire)

    def callback_expire(self, response):
        # récupération de la date
        expire = ""
        for line in response.split('\n'):
            line = line[13:].strip()
            if line.startswith('next'):
                expire = line.split()[1:5]
                expire = " ".join(expire).split(',')[0]
        # transformation en datetime
        if expire == "":
            self.status = status.Warn("date d'invalidation non trouvée")
            return [{'expire':"ipsec listcrls ne renvoie pas de date d'invalidation de la crl" }]
        expdate = util.parse_date(expire)
        # calcul de la date avant laquelle on est en warning (expiration - 20 minutes)
        warn_delta = datetime.timedelta(seconds=60*20)
        warndate = expdate - warn_delta
        date = datetime.datetime.utcnow()
        if date > expdate:
            msg = "expiration de la CRL"
            self.status = status.Error(msg)
        else:
            delta = expdate - date
            minutes = (delta.days*3600*24 + delta.seconds) / 60
            if date > warndate:
                self.status=status.Warn("Expiration dans %s minutes" % minutes)
        return [{'expire': expdate.strftime('%d %b %Y %H:%M:%S')}]

    def errback_expire(self, err):
        msg = u"erreur de lecture de la date d'expiration"
        self.status = status.Warn(msg)
        return None

    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value

    def check_status(self):
        return self.status

