#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2013
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# service_restart.py
#
# code effectif de la relance d'un service par Zéphir
# à lancer par service_restart.zephir <nom_service>
###########################################################################

import os, sys, json
from zephir.lib_zephir import *
from creole import config
from pyeole.process import system_code
from pyeole.service import service_code_no_container, get_containers_group

def creole_service_conf(service):
    service_conf = json.load(file(config.init_services_file))
    if service not in service_conf:
        return None
    containers = service_conf.get(service, ('service', False, []))[2]
    return get_containers_group(containers)

def do_restart(service, services_ok):
    """redémarre le service demandé. Si nécessaire redémarre les instances
    du service lancés dans les conteneurs
    """
    if not is_locked(['reconfigure','maj','configure','sauvegarde']):
        lock(['reconfigure'])
        cont_ok = []
        cont_err = []
        in_container = False
        try:
            if service == "rvp":
                # cas particulier : rvp, nécessite de redémarrer named
                service_code_no_container('rvp', 'stop')
                service_code_no_container('bind9', 'stop')
                service_code_no_container('rvp', 'start')
                res = service_code_no_container('bind9', 'start')
                if res == 0:
                    cont_ok = ['root']
                else:
                    cont_err = ['root']
            else:
                # dans le cas d'un service 'eole', on regarde dans quels conteneurs il est activé
                creole_containers = creole_service_conf(service)
                for serv_script in services_ok:
                    cmd_reconf = ["/usr/sbin/service", service, "restart"]
                    if serv_script.startswith(config.VIRTROOT):
                        in_container = True
                        # recherche du nom de conteneur dans le chemin
                        conteneur = serv_script.replace('%s/' % config.VIRTROOT, '')
                        conteneur = conteneur.split(os.sep)[0]
                        if creole_containers is None or conteneur in creole_containers:
                            # appel de la commande dans le conteneur si activé dans creole
                            # ou si non géré par creole
                            if system_code(cmd_reconf, container=conteneur) == 0:
                                cont_ok.append(conteneur)
                            else:
                                cont_err.append(conteneur)
                    elif creole_containers is None or 'root' in creole_containers:
                        if system_code(cmd_reconf) == 0:
                            cont_ok.append('root')
                        else:
                            cont_err.append('root')
        except Exception, e:
            log('SERVICE_RESTART', 1, 'Erreur interne lors du redémarrage du service %s : %s' % (service, str(e)))
            unlock(['reconfigure'])
            exit(1)
        unlock(['reconfigure'])
        if in_container:
            msg_container = " dans le(s) conteneur(s): %s" % ",".join(cont_ok)
        else:
            msg_container = ""
        log('SERVICE_RESTART', 0, 'service %s relancé%s' % (service, msg_container))
        # génération du message de retour suivant le résultat dans chaque conteneur
        if cont_err:
            if in_container:
                msg_err_container = " dans le(s) conteneur(s): %s" % ",".join(cont_err)
            else:
                msg_err_container = ""
            log('SERVICE_RESTART', 1, 'Erreur lors du redémarrage du service %s%s' % (service, msg_err_container))
    else:
        # une tache lancée par zephir n'est pas (ou mal) terminée
        # on n'autorise pas reconfigure dans cet état
        log('SERVICE_RESTART',1,"""une tache concurrente est en exécution: verrou dans /var/lock/""")

if __name__ == '__main__':
    try:
        service = sys.argv[1]
        services_ok = sys.argv[2:]
    except:
        print "Erreur, arguments manquants"
    do_restart(service, services_ok)
